local MAJOR = "StarLibWidgetKey-1.0" 
local MINOR = 13

assert(LibStub, MAJOR.." requires LibStub") 
local WidgetKey = LibStub:NewLibrary(MAJOR, MINOR)
if not WidgetKey then return end
local LibTimer = LibStub("StarLibTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires StarLibTimer-1.0")
local Evaluator = LibStub("StarLibEvaluator-1.0", true)
assert(Evaluator, MAJOR .. " requires StarLibEvaluator-1.0")

local pool = setmetatable({}, {__mode = "k"})

WidgetKey.defaults = {
	expression = 'return',
	up = false,
	modifier = 1,
	simple = true
}

local widgetType = {key=true}

-- Create a new StarLibWidgetKey object
-- @name WidgetKey.New
-- @usage WidgetKey:New(visitor, name, config, row, col, layer, fontString, env, errorLevel)
-- @param visitor An StarLibCore-1.0 object
-- @param config This widget's settings
-- @param row This widget's row
-- @param col This widget's column
-- @param layer This widget's layer
-- @param config The settings. You can provide these fields: string, prefix, postfix, precision, align, update, scroll, speed, direction, cols, offset, str, update, speed. Check the source for further details. Some fields have defaults.
-- @param self.errorLevel The self.errorLevel for this object
-- @param callback Your draw function. The widget is passed as first parameter and data is passed as 2nd.
-- @param data Some data to pass to the draw function
-- @return A new StarLibWidgetKey object

function WidgetKey:New(visitor, name, config, errorLevel, callback, data) 
	assert(name, "WidgetKey requires a name.")
	assert(config, "Please provide the timer with a config")
	assert(config.expression, name .. ": Please provide the marquee with a string")
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
		obj.options = {}
	end
		
	setmetatable(obj, self)

	obj.widget = LibWidget:New(obj, visitor, name, config, 0, 0, 0, widgetType, errorLevel)
	
	obj.config = config
	obj.expression = config.expression or defaults.expression
	obj.up = config.up or defaults.up
	obj.modifier = config.modifier or defaults.modifier
	obj.simple = config.simple or defaults.simple
	
	
	return obj	
end


-- Delete a StarLibWidgetKey object
-- @name StarLibWidgetKey.Del
-- @usage StarLibWidgetKey:Del([marq])
-- @param marq A StarLibWidgetKey object
function WidgetKey:Del(key)
	if not key then
		key = self
	end
	key.widget:Del()
	key.widget = nil
	key:Stop()
	pool[key] = true
end

-- Start a StarLibWidgetKey
-- @name StarLibWidgetKey.Start
-- @usage object:Start()
-- @return Nothing
function WidgetKey:Start()
	self.error:Print("WidgetKey:Start")
	self.enabled = true
end

-- Stop a StarLibWidgetKey
-- @name StarLibWidgetKey.Stop
-- @usage object:Stop()
-- @return Nothing
function WidgetKey:Stop()
	self.error:Print("WidgetKey:Stop")
	self.enabled = false
end

function WidgetKey:KeyEvent(modifier, up)
	if not self.enabled then return end
	local mod = modifier
	if self.modifier == 1 and self.simple then
		mod = (modifier == "LCTRL" or modifier == "RCTRL") and "LCTRL"
		modifier = "LCTRL"
	elseif self.modifier == 2 and self.simple then
		mod = (modifier == "LALT" or modifier == "RALT") and "LALT"
		modifier = "LALT"
	elseif self.modifier == 3 and self.simple then
		mod = (modifier == "LSHIFT" or modifier == "RSHIFT") and "LSHIFT"
		modifier = "LSHIFT"
	end
		
	if mod == modifier and self.up == up then
		self.visitor.environment.self = self
		Evaluator.ExecuteCode(self.visitor.environment, self.name, self.expression)
	end
end

function WidgetKey:GetOptions(visitor, db)
		local defaults = WidgetKey.defaults
		local options = {
			expression = {
				name = "Expression",
				desc = "Enter this widget's expression",
				type = "input",
				width = "full",
				multiline = true,
				get = function()
					return db.expression or defaults.expression
				end,
				set = function(info, v)
					db.expression = v
				end,
				order = 3
			},
			
		}
	return options
end
